function getUniqueValues(count, min, max) {
    const values = [];
    while (values.length < count) {
        const randomValue = Math.floor(Math.random() * (max - values.length - min)) + min;
        values.push(values.slice().sort((a, b) => a - b).reduce((newValue, oldValue) => ((newValue >= oldValue) ? (newValue + 1) : newValue), randomValue));
    }
    return values;
}

function randomInt(min, max) {
    return Math.floor(Math.random() * (max - min + 1)) + min;
}

function getDigit(allowZero) {
    if (allowZero && Math.random() < 0.5)
        return 0;
    return Math.floor(Math.random() * 9) + 1;
}

function randomUnit() {
    const prefix = ['', 'k', 'm'];
    const units = ['m', 'g', 'l', 'A', 'V', 'm²', 'm³', 'Pa'];
    return prefix[randomInt(0, prefix.length - 1)] + units[randomInt(0, units.length - 1)];
}

function addQuestion(arrFragen, masszahl, gz) {
    const frage = {
        "html": "Wie viele gültige Ziffern hat <code>" + masszahl + "</code>?",
        "keepOrder": true,
        "singlechoice": true,
        "choices": [
            { "html": "", "unit": " gültige Ziffern", "type": "number", size: 3, min: 0, max: 99, step: 1, "selectalways": true, correct: true, accept: gz }
        ]
    };
    arrFragen.push(frage);
}

function generateSimple(arrFragen) {
    const gz = randomInt(1, 7);
    const comma = randomInt(0, gz - 1);
    let v = '';
    while (v.length < gz)
        v += '' + getDigit(v.length && v.length != gz - 1);
    v /= Math.pow(10, comma);
    v = ('' + v).replace('.', ',');
    addQuestion(arrFragen, v + ' ' + randomUnit(), gz);
}

function generateExtraZeros(arrFragen) {
    const gz = randomInt(1, 7);
    const comma = randomInt(0, gz - 1);
    let v = '';
    while (v.length < gz)
        v += '' + getDigit(v.length && v.length != gz - 1);
    v /= Math.pow(10, comma);
    v = ('' + v).replace('.', ',');
    const zeros = randomInt(1, 9 - gz);
    if (v.indexOf(',') == -1)
        v += ',';
    v += '0'.repeat(zeros);
    addQuestion(arrFragen, v + ' ' + randomUnit(), gz + zeros);
}

function generateLeadingZeros(arrFragen) {
    const gz = randomInt(1, 7);
    const pre = '0,' + '0'.repeat(randomInt(1, 5));
    let v = '';
    while (v.length < gz)
        v += '' + getDigit(v.length && v.length != gz - 1);
    addQuestion(arrFragen, pre + v + ' ' + randomUnit(), gz);
}

window.MultipleChoiceTestData = {
    // Optional: Wie viele Versuche hat der/die Lernende (Standardwert: Unendlich)
    "maxTries": Infinity,

    // Optional: Soll statt der Anzahl an nötigen Versuchen bewertet werden, wie gut der/die Lernende abschneidet?
    "reportScore": false,

    // Optional: Sollen die Fragen nicht durchgemischt werden?
    "keepOrder": false,

    // Optional: Wie soll bewertet werden?
    // "questions"  → Standardwert, Frage kann entweder richtig sein (1 von 1 Punkt) oder falsch (0 von 1 Punkt)
    // "average"    → Durchschnittlicher Prozentsatz an richtig ausgewählten bzw. richtig nicht ausgewählten Antwortmöglichkeiten pro auswählbaren Antwortmöglichkeiten bei jeder Frage
    // "choices"    → Anteil der richtig ausgewählten Antwortmöglichkeiten pro auswählbaren Antwortmöglichkeiten
    "countMode": "questions",

    // Wie viele Fragen sollen für einen Test gezogen werden?
    "questionsPerTest": 5,

    // Welcher Anteil muss mindestens richtig sein, um zu bestehen?
    "threshold": 1,

    // Optional: Begrüßungstext
    "htmlIntro": "<p>Trage je ein, wie viele gültige Ziffern die Größe hat:</p>",

    // Verfügbare Fragen
    "questions": [
    ],

    // Fragengeneratoren
    "generators": [
        generateSimple,
        generateSimple,
        generateLeadingZeros,
        generateLeadingZeros,
        generateExtraZeros,
        generateExtraZeros
    ]
}
